@@ -6,28 +6,22 @@ from django.db import models as django_models
 
 
 def default_cache_func(
-        django_model,
         redis_root,
+        django_model,
         redis_model,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
-        filter_by,
+        cache_conf,
 ):
     redis_dicts = redis_root.get(redis_model, return_dict=True)
     cache_to_django(
+        redis_root,
         django_model,
         redis_dicts,
-        redis_root,
     )
     django_to_cache(
+        redis_root,
         django_model,
         redis_model,
-        redis_root,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
-        filter_by,
+        cache_conf,
     )
 
 
@@ -60,6 +54,7 @@ async def update_or_create_django_instance_from_redis_instance(
         redis_instance,
         django_model,
 ):
+    print(redis_instance['id'], redis_instance['django_id'])
     django_id = redis_instance['django_id']
     django_params, django_many_to_many_params = await async_redis_dict_to_django_params(
         redis_instance,
@@ -255,23 +250,18 @@ def django_sync_to_async_create(
 
 
 def django_to_cache(
+        redis_root,
         django_model,
         redis_model,
-        redis_root,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
-        filter_by,
+        cache_conf,
 ):
     async def run_django_to_cache(
+            redis_root,
             django_model,
             redis_model,
-            redis_root,
-            save_related_models,
-            get_or_create_redis_model_from_django_model,
-            exclude_fields,
-            filter_by,
+            cache_conf,
     ):
+        filter_by = cache_conf['filter_by']
         if filter_by.keys():
             if django_model in filter_by.keys():
                 django_instances = await django_sync_to_async_list(django_model.objects.filter, filter_by[django_model])
@@ -287,12 +277,10 @@ def django_to_cache(
         for django_instance in django_instances:
             async_tasks.append(
                 update_or_create_redis_instance_from_django_instance(
+                    redis_root,
                     django_instance,
                     redis_model,
-                    redis_root,
-                    save_related_models,
-                    get_or_create_redis_model_from_django_model,
-                    exclude_fields,
+                    cache_conf,
                 )
             )
             completed += 1
@@ -305,41 +293,34 @@ def django_to_cache(
     loop = asyncio.get_event_loop()
     loop.run_until_complete(
         run_django_to_cache(
+            redis_root,
             django_model,
             redis_model,
-            redis_root,
-            save_related_models,
-            get_or_create_redis_model_from_django_model,
-            exclude_fields,
-            filter_by,
+            cache_conf,
         )
     )
 
 
 async def update_or_create_redis_instance_from_django_instance(
+        redis_root,
         django_instance,
         redis_model,
-        redis_root,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
+        cache_conf,
 ):
-    redis_instance = redis_root.get(redis_model, django_id=django_instance.id)
+    django_instance_model = django_instance.__class__
+    redis_instance = redis_root.get(django_instance_model, django_id=django_instance.id)
     redis_dict = await django_instance_to_redis_params(
-        django_instance,
         redis_root,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
+        django_instance,
+        cache_conf,
     )
 
     if not redis_instance:
         fields_to_create = redis_dict
-        redis_instance = redis_model(
-            redis_root=redis_root,
+        redis_instance = redis_root.create(
             django_id=django_instance.id,
             **fields_to_create
-        ).save()
+        )
         result_redis_instance_id = redis_instance['id']
     else:
         redis_instance = redis_instance[0]
@@ -347,7 +328,7 @@ async def update_or_create_redis_instance_from_django_instance(
             redis_instance,
             redis_dict,
         )
-        redis_root.update(redis_model, redis_instance,
+        redis_root.update(django_instance_model, redis_instance,
                           **fields_to_update
                           )
         result_redis_instance_id = redis_instance['id']
@@ -370,13 +351,12 @@ def check_fields_need_to_update(
 
 
 async def django_instance_field_to_redis_value(
+        redis_root,
         django_instance,
         django_field,
-        redis_root,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
+        cache_conf,
 ):
+    save_related_models = cache_conf['save_related_models']
     django_field_value = await django_sync_to_async_get(django_instance, django_field.name)
     redis_value = django_field_value
     if django_field.__class__ == django_models.ForeignKey:
@@ -388,9 +368,7 @@ async def django_instance_field_to_redis_value(
                 redis_foreign_key_instance = await get_or_create_redis_instance_from_django_instance(
                     django_foreign_key_instance,
                     redis_root,
-                    save_related_models,
-                    get_or_create_redis_model_from_django_model,
-                    exclude_fields,
+                    cache_conf,
                 )
                 redis_value = redis_foreign_key_instance
             else:
@@ -402,9 +380,7 @@ async def django_instance_field_to_redis_value(
                 await get_or_create_redis_instance_from_django_instance(
                     django_many_to_many_instance,
                     redis_root,
-                    save_related_models,
-                    get_or_create_redis_model_from_django_model,
-                    exclude_fields,
+                    cache_conf,
                 )
                 for django_many_to_many_instance in django_many_to_many_instances
             ]
@@ -422,14 +398,13 @@ async def django_instance_field_to_redis_value(
 
 
 async def django_instance_to_redis_params(
-        django_instance,
         redis_root,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
+        django_instance,
+        cache_conf,
 ):
     redis_params = {}
     django_model = django_instance.__class__
+    exclude_fields = cache_conf['exclude_fields']
     for i, django_field in enumerate(django_instance.__class__._meta.get_fields()):
         if not django_field.__class__.__name__.endswith('Rel'):
             django_field_name = django_field.name
@@ -439,12 +414,10 @@ async def django_instance_to_redis_params(
             if allowed:
                 if django_field.name not in ['id', 'pk']:
                     redis_param = await django_instance_field_to_redis_value(
+                        redis_root,
                         django_instance,
                         django_field,
-                        redis_root,
-                        save_related_models,
-                        get_or_create_redis_model_from_django_model,
-                        exclude_fields,
+                        cache_conf,
                     )
                     if redis_param:
                         redis_params[django_field_name] = redis_param
@@ -454,28 +427,18 @@ async def django_instance_to_redis_params(
 async def get_or_create_redis_instance_from_django_instance(
         django_instance,
         redis_root,
-        save_related_models,
-        get_or_create_redis_model_from_django_model,
-        exclude_fields,
+        cache_conf,
 ):
     django_instance_model = django_instance.__class__
-    redis_model = get_or_create_redis_model_from_django_model(
-        django_instance_model,
-        redis_root,
-        save_related_models,
-        exclude_fields,
-    )
-    redis_instance = redis_root.get(redis_model, django_id=django_instance.id)
+    redis_instance = redis_root.get(django_instance_model, django_id=django_instance.id)
     if not redis_instance:
         redis_dict = await django_instance_to_redis_params(
-            django_instance,
             redis_root,
-            save_related_models,
-            get_or_create_redis_model_from_django_model,
-            exclude_fields,
+            django_instance,
+            cache_conf,
         )
-        redis_instance = redis_model(
-            redis_root=redis_root,
+        redis_instance = redis_root.create(
+            django_model=django_instance_model,
             django_id=django_instance.id,
             **redis_dict
         ).save()
