@@ -155,7 +155,6 @@ class RedisForeignKey(RedisNumber):
     def clean(self):
         self.value = self.check_value()
         if self.value not in [None, 'null']:
-            print(get_ids_from_untyped_data(self.value)[0])
             self.value = get_ids_from_untyped_data(self.value)[0]
         return super().clean()
 
@@ -327,18 +326,18 @@ class RedisDate(RedisString):
 
 # class RedisDjangoForeignKey(RedisNumber):
 #
-#     def __init__(self, model=None, **kwargs):
+#     def __init__(self, django_model=None, **kwargs):
 #         super().__init__(**kwargs)
-#         self.model = None
-#         if not issubclass(model.__class__, django_models.Model):
-#             raise Exception(f'{model.__class__.__name__} class is not django.db.models Model')
-#         self.model = model
+#         self.django_model = None
+#         if not issubclass(django_model.__class__, django_models.Model):
+#             raise Exception(f'{django_model.__class__.__name__} class is not django.db.models Model')
+#         self.django_model = django_model
 #
 #     def _get_id_from_instance(self):
 #         if issubclass(self.value.__class__, django_models.Model):
-#             if isinstance(self.value, self.model):
+#             if isinstance(self.value, self.django_model):
 #                 if self.value.id:
-#                     django_instances_with_this_id = self.model.objects.filter(id=self.value.id)
+#                     django_instances_with_this_id = self.django_model.objects.filter(id=self.value.id)
 #                     if django_instances_with_this_id:
 #                         if len(django_instances_with_this_id) == 1:
 #                             self.value = self.value.id
@@ -351,7 +350,7 @@ class RedisDate(RedisString):
 #                 else:
 #                     raise Exception(f'This django instance ({self.value}) has no attribute id')
 #             else:
-#                 raise Exception(f'{self.value} is not instance of {self.model}')
+#                 raise Exception(f'{self.value} is not instance of {self.django_model}')
 #         else:
 #             raise Exception(f'{self.value} class is not django.db.models Model')
 #         return self.value
@@ -366,7 +365,7 @@ class RedisDate(RedisString):
 #         instance = {
 #             'id': id
 #         }
-#         django_instances_with_this_id = self.model.objects.filter(id=id)
+#         django_instances_with_this_id = self.django_model.objects.filter(id=id)
 #         if django_instances_with_this_id:
 #             if len(django_instances_with_this_id) == 1:
 #                 instance = django_instances_with_this_id[0]
@@ -438,6 +437,140 @@ class RedisRoot:
             print(f"{datetime.datetime.now()} - {', '.join([str(model) for model in django_models_to_cache.keys()])} will be cached now...")
         return django_models_to_cache
 
+    def get_cache_conf(self, django_model):
+        if django_model in self.registered_django_models.keys():
+            cache_conf = self.registered_django_models[django_model]
+            return cache_conf
+        else:
+            raise Exception(f'{django_model} is not registered')
+
+    def get_or_create_redis_model_from_django_model(self, django_model):
+
+        def django_fields_to_redis_fields(django_model, redis_root, save_related_models, exclude_fields):
+            redis_fields = {}
+            for django_field in django_model._meta.get_fields():
+                django_field_name = django_field.name
+                allowed = True
+                if django_model in exclude_fields.keys():
+                    allowed = (django_field_name not in exclude_fields[django_model])
+                if allowed:
+                    if django_field.name in ['id', 'pk']:
+                        django_field_name = 'django_id'
+                        redis_field = RedisNumber(null=False)
+                    else:
+                        redis_field = django_field_to_redis_field(django_field, redis_root, save_related_models,
+                                                                  exclude_fields)
+                    if redis_field:
+                        redis_fields[django_field_name] = redis_field
+            return redis_fields
+
+        def create_redis_model_from_redis_fields(django_model, redis_fields, redis_root):
+            new_redis_model = type(django_model.__name__, (RedisModel,), redis_fields)
+            redis_root.register_models([new_redis_model])
+            return new_redis_model
+
+        def django_field_to_redis_field(django_field, redis_root, save_related_models, exclude_fields):
+
+            field_mapping = {
+                django_models.BooleanField: RedisBool,
+                django_models.DecimalField: RedisDecimal,
+                django_models.PositiveSmallIntegerField: RedisNumber,
+                django_models.BigAutoField: RedisNumber,
+                django_models.BigIntegerField: RedisNumber,
+                django_models.DateField: RedisDate,
+                django_models.DateTimeField: RedisDateTime,
+                django_models.EmailField: RedisString,
+                django_models.IntegerField: RedisNumber,
+                django_models.GenericIPAddressField: RedisString,
+                django_models.JSONField: RedisJson,
+                django_models.PositiveBigIntegerField: RedisNumber,
+                django_models.PositiveIntegerField: RedisNumber,
+                django_models.SmallAutoField: RedisNumber,
+                django_models.SmallIntegerField: RedisNumber,
+                django_models.TextField: RedisString,
+                django_models.URLField: RedisString,
+                django_models.UUIDField: RedisString,
+                django_models.ForeignKey: RedisForeignKey,
+                django_models.ManyToManyField: RedisManyToMany
+            }
+
+            if django_field.__class__.__name__.endswith('Rel'):
+                redis_field = None
+            elif django_field.__class__ in field_mapping.keys():
+                redis_field = field_mapping[django_field.__class__]
+            else:
+                redis_field = RedisString
+
+            if redis_field:
+                redis_field_class = type(
+                    django_field.__class__.__name__,
+                    (redis_field,),
+                    {}
+                )
+
+                redis_field_params = django_field_params_to_redis_field_params(django_field, redis_root,
+                                                                               save_related_models, exclude_fields)
+                redis_field = redis_field_class(**redis_field_params)
+            return redis_field
+
+        def django_field_params_to_redis_field_params(django_field, redis_root, save_related_models, exclude_fields):
+            allowed_params = ['null', 'default', 'choices', 'remote_field']
+            allowed_redis_params = {}
+            for django_field_param_name in django_field.__dict__.keys():
+                redis_field_param_name = django_field_param_name
+                if django_field_param_name in allowed_params:
+                    value = getattr(django_field, django_field_param_name)
+                    if value in [django_models.NOT_PROVIDED, django_models.Empty, django_models.BLANK_CHOICE_DASH]:
+                        value = None
+                    if django_field_param_name == 'choices':
+                        real_value = {}
+                        if value is not None:
+                            for choice in value:
+                                real_value[choice[0]] = choice[1]
+                        else:
+                            real_value = None
+                        value = real_value
+                        allowed_redis_params[redis_field_param_name] = value
+                    elif django_field_param_name == 'remote_field':
+                        redis_field_param_name = 'django_model'
+                        if value is not None:
+                            value = value.model
+                            if value is not None:
+                                value = get_or_create_redis_model_from_django_model(value, redis_root,
+                                                                                    save_related_models, exclude_fields)
+                                if value is not None:
+                                    allowed_redis_params[redis_field_param_name] = value
+                                    allowed_redis_params['save_related_models'] = save_related_models
+                    else:
+                        allowed_redis_params[redis_field_param_name] = value
+            return allowed_redis_params
+
+        def update_redis_model_from_redis_fields(django_model, existing_redis_model, redis_fields, redis_root):
+            new_redis_model = create_redis_model_from_redis_fields(django_model, redis_fields, redis_root)
+            new_registered_models = []
+            for redis_model in redis_root.registered_models:
+                if redis_model == existing_redis_model:
+                    new_registered_models.append(new_redis_model)
+                else:
+                    new_registered_models.append(redis_model)
+            redis_root.registered_models = new_registered_models
+            return new_redis_model
+
+        cache_conf = self.check_cache_conf(django_model)
+        save_related_models = cache_conf['save_related_models']
+        exclude_fields = cache_conf['exclude_fields']
+
+        redis_fields = django_fields_to_redis_fields(django_model, self, save_related_models, exclude_fields)
+        registered_models_names = [model.__name__ for model in self.registered_models]
+        if django_model.__name__ in registered_models_names:
+            existing_redis_model_index = registered_models_names.index(django_model.__name__)
+            existing_redis_model = self.registered_models[existing_redis_model_index]
+            new_redis_model = update_redis_model_from_redis_fields(django_model, existing_redis_model, redis_fields,
+                                                                   self)
+        else:
+            new_redis_model = create_redis_model_from_redis_fields(django_model, redis_fields, self)
+        return new_redis_model
+
     def check_cache(self):
         django_models_to_cache = self._get_django_models_to_cache()
         exclude_fields = {
@@ -451,135 +584,14 @@ class RedisRoot:
         for django_model, cache_conf in django_models_to_cache.items():
             cache_func = cache_conf['cache_func']
             save_related_models = cache_conf['save_related_models']
-            
-            def get_or_create_redis_model_from_django_model(django_model, redis_root, save_related_models, exclude_fields):
-                registered_models_names = [model.__name__ for model in redis_root.registered_models]
-                redis_fields = django_fields_to_redis_fields(django_model, redis_root, save_related_models, exclude_fields)
-                if django_model.__name__ in registered_models_names:
-                    existing_redis_model_index = registered_models_names.index(django_model.__name__)
-                    existing_redis_model = redis_root.registered_models[existing_redis_model_index]
-                    new_redis_model = update_redis_model_from_redis_fields(django_model, existing_redis_model, redis_fields, redis_root)
-                else:
-                    new_redis_model = create_redis_model_from_redis_fields(django_model, redis_fields, redis_root)
-                return new_redis_model
-            
-            def django_fields_to_redis_fields(django_model, redis_root, save_related_models, exclude_fields):
-                redis_fields = {}
-                for django_field in django_model._meta.get_fields():
-                    django_field_name = django_field.name
-                    allowed = True
-                    if django_model in exclude_fields.keys():
-                        allowed = (django_field_name not in exclude_fields[django_model])
-                    if allowed:
-                        if django_field.name in ['id', 'pk']:
-                            django_field_name = 'django_id'
-                            redis_field = RedisNumber(null=False)
-                        else:
-                            redis_field = django_field_to_redis_field(django_field, redis_root, save_related_models, exclude_fields)
-                        if redis_field:
-                            redis_fields[django_field_name] = redis_field
-                return redis_fields
 
-            def create_redis_model_from_redis_fields(django_model, redis_fields, redis_root):
-                new_redis_model = type(django_model.__name__, (RedisModel,), redis_fields)
-                redis_root.register_models([new_redis_model])
-                return new_redis_model
-
-            def django_field_to_redis_field(django_field, redis_root, save_related_models, exclude_fields):
-
-                field_mapping = {
-                    django_models.BooleanField: RedisBool,
-                    django_models.DecimalField: RedisDecimal,
-                    django_models.PositiveSmallIntegerField: RedisNumber,
-                    django_models.BigAutoField: RedisNumber,
-                    django_models.BigIntegerField: RedisNumber,
-                    django_models.DateField: RedisDate,
-                    django_models.DateTimeField: RedisDateTime,
-                    django_models.EmailField: RedisString,
-                    django_models.IntegerField: RedisNumber,
-                    django_models.GenericIPAddressField: RedisString,
-                    django_models.JSONField: RedisJson,
-                    django_models.PositiveBigIntegerField: RedisNumber,
-                    django_models.PositiveIntegerField: RedisNumber,
-                    django_models.SmallAutoField: RedisNumber,
-                    django_models.SmallIntegerField: RedisNumber,
-                    django_models.TextField: RedisString,
-                    django_models.URLField: RedisString,
-                    django_models.UUIDField: RedisString,
-                    django_models.ForeignKey: RedisForeignKey,
-                    django_models.ManyToManyField: RedisManyToMany
-                }
-
-                if django_field.__class__.__name__.endswith('Rel'):
-                    redis_field = None
-                elif django_field.__class__ in field_mapping.keys():
-                    redis_field = field_mapping[django_field.__class__]
-                else:
-                    redis_field = RedisString
-
-                if redis_field:
-                    redis_field_class = type(
-                        django_field.__class__.__name__,
-                        (redis_field,),
-                        {}
-                    )
-
-                    redis_field_params = django_field_params_to_redis_field_params(django_field, redis_root, save_related_models, exclude_fields)
-                    redis_field = redis_field_class(**redis_field_params)
-                return redis_field
-
-            def django_field_params_to_redis_field_params(django_field, redis_root, save_related_models, exclude_fields):
-                allowed_params = ['null', 'default', 'choices', 'remote_field']
-                allowed_redis_params = {}
-                for django_field_param_name in django_field.__dict__.keys():
-                    redis_field_param_name = django_field_param_name
-                    if django_field_param_name in allowed_params:
-                        value = getattr(django_field, django_field_param_name)
-                        if value in [django_models.NOT_PROVIDED, django_models.Empty, django_models.BLANK_CHOICE_DASH]:
-                            value = None
-                        if django_field_param_name == 'choices':
-                            real_value = {}
-                            if value is not None:
-                                for choice in value:
-                                    real_value[choice[0]] = choice[1]
-                            else:
-                                real_value = None
-                            value = real_value
-                            allowed_redis_params[redis_field_param_name] = value
-                        elif django_field_param_name == 'remote_field':
-                            redis_field_param_name = 'model'
-                            if value is not None:
-                                value = value.model
-                                if value is not None:
-                                    value = get_or_create_redis_model_from_django_model(value, redis_root, save_related_models, exclude_fields)
-                                    if value is not None:
-                                        allowed_redis_params[redis_field_param_name] = value
-                                        allowed_redis_params['save_related_models'] = save_related_models
-                        else:
-                            allowed_redis_params[redis_field_param_name] = value
-                return allowed_redis_params
-
-            def update_redis_model_from_redis_fields(django_model, existing_redis_model, redis_fields, redis_root):
-                new_redis_model = create_redis_model_from_redis_fields(django_model, redis_fields, redis_root)
-                new_registered_models = []
-                for redis_model in redis_root.registered_models:
-                    if redis_model == existing_redis_model:
-                        new_registered_models.append(new_redis_model)
-                    else:
-                        new_registered_models.append(redis_model)
-                redis_root.registered_models = new_registered_models
-                return new_redis_model
-
-            redis_model = get_or_create_redis_model_from_django_model(django_model, self, save_related_models, exclude_fields)
+            redis_model = self.get_or_create_redis_model_from_django_model(django_model)
 
             cache_func(
-                django_model,
                 self,
+                django_model,
                 redis_model,
-                save_related_models,
-                get_or_create_redis_model_from_django_model,
-                exclude_fields,
-                filter_by,
+                cache_conf,
             )
             print(f'{datetime.datetime.now()} - Cached {django_model} successfully')
 
@@ -619,7 +631,7 @@ class RedisRoot:
             else:
                 raise Exception(f'{model.__name__} class is not RedisModel')
 
-    def get_cache_conf(self, model, user_cache_conf=None):
+    def check_cache_conf(self, model, user_cache_conf=None):
         name = model.__name__
 
         cache_conf = {
@@ -684,7 +696,7 @@ class RedisRoot:
         for model, user_cache_conf in models_dict.items():
             if issubclass(model, django_models.Model):
                 if model not in self.registered_django_models.keys():
-                    cache_conf = self.get_cache_conf(model, user_cache_conf)
+                    cache_conf = self.check_cache_conf(model, user_cache_conf)
                     self.registered_django_models[model] = {
                         'cache_conf': cache_conf,
                         'to_cache_in': datetime.datetime.now()
@@ -854,7 +866,6 @@ class RedisRoot:
                 instances[instance_id][field_name] = value
         return instances
 
-
     def _get_all_stored_model_instances(self, model, filters=None):
         model_name = model.__name__
         instances = self._get_instances_by_key(f'{self.prefix}:{model_name}:*')
@@ -892,8 +903,17 @@ class RedisRoot:
             instances_with_allowed = self._check_fields_existence(model, instances_with_allowed, filters)
         instances = self._get_instances_from_instances_with_allowed(instances_with_allowed)
         return instances
+    
+    def _django_model_to_redis_model(self, django_model):
+        if django_model in self.registered_django_models.keys():
+            redis_model = self.get_or_create_redis_model_from_django_model(django_model)
+            return redis_model
+        else:
+            raise Exception(f'{django_model} is not registered in redis cache')
 
-    def get(self, model, return_dict=False, **filters):
+
+    def get(self, django_model, return_dict=False, **filters):
+        model = self._django_model_to_redis_model(django_model)
         instances = self._get_all_model_instances(model, filters)
         return self._return_with_format(instances, return_dict)
 
@@ -942,12 +962,13 @@ class RedisRoot:
             fields_to_write_json = json.dumps(fields_to_write)
             self.redis_instance.set(instance_key, fields_to_write_json, ex=ttl)
             updated_data = {
-                'model': saved_model,
+                'django_model': saved_model,
                 'id': instance_id
             }
         return updated_data
 
-    def update(self, model, instances=None, return_dict=False, renew_ttl=False, new_ttl=None, **fields_to_update):
+    def update(self, django_model, instances=None, return_dict=False, renew_ttl=False, new_ttl=None, **fields_to_update):
+        model = self._django_model_to_redis_model(django_model)
         model_name = model.__name__
         updated_datas = []
         if instances is None:
@@ -964,7 +985,7 @@ class RedisRoot:
                         updated_datas.append(updated_data)
         updated_instances = {}
         for updated_data in updated_datas:
-            updated_model = updated_data['model']
+            updated_model = updated_data['django_model']
             updated_id = updated_data['id']
             if self.economy:
                 updated_instances[updated_id] = {'id': updated_id}
@@ -977,7 +998,8 @@ class RedisRoot:
     def _delete_by_key(self, key):
         self.redis_instance.delete(key)
 
-    def delete(self, model, instances=None):
+    def delete(self, django_model, instances=None):
+        model = self._django_model_to_redis_model(django_model)
         model_name = model.__name__
         if instances is None:
             for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:*'):
@@ -988,8 +1010,8 @@ class RedisRoot:
                 for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:{instance_id}'):
                     self._delete_by_key(key)
 
-    # def get_id_from_django_id(self, model, django_id):
-    #     model_name = model.__name__
+    # def get_id_from_django_id(self, django_model, django_id):
+    #     model_name = django_model.__name__
     #     check_types(django_id, int)
     #     result = False
     #     for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:*:django_id'):
@@ -999,8 +1021,8 @@ class RedisRoot:
     #             result = instance_id
     #     return result
     #
-    # def is_instance_edited_by_id(self, model, instance_id):
-    #     model_name = model.__name__
+    # def is_instance_edited_by_id(self, django_model, instance_id):
+    #     model_name = django_model.__name__
     #     check_types(instance_id, int)
     #     result = False
     #     for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:{instance_id}:edited'):
@@ -1009,6 +1031,15 @@ class RedisRoot:
     #         result = value
     #     return result
 
+    def create(self, django_model, **params):
+        model = self._django_model_to_redis_model(django_model)
+        redis_instance_id = model(redis_root=self, **params).save()
+        if self.economy:
+            redis_instances = {'id': redis_instance_id}
+        else:
+            redis_instance = self.get(django_model, id=redis_instance_id)
+        return redis_instance
+
 
 class RedisModel:
     id = RedisId()
@@ -1105,10 +1136,15 @@ class RedisModel:
         data_to_write = self._serialize_data()
         model_ttl = self.get_model_ttl()
         redis_root = self.__model_data__['redis_root']
+        saved_instance = {}
         for instance_key, fields_dict in data_to_write.items():
+            prefix, model_name, instance_id = instance_key.split(':')
             redis_instance = redis_root.redis_instance
             fields_json = json.dumps(fields_dict)
             redis_instance.set(instance_key, fields_json, ex=model_ttl)
+            if instance_id not in saved_instance.keys():
+                saved_instance[instance_id] = fields_dict
+        return saved_instance
 
     def _get_new_id(self):
         redis_root = self.__model_data__['redis_root']
@@ -1121,13 +1157,8 @@ class RedisModel:
         self.id.value = int(max_id + 1)
 
     def save(self):
-        self._set_fields()
-        redis_root = self.__model_data__['redis_root']
-        if self.__model_data__['redis_root'].economy:
-            deserialized_instance = {'id': self.id.value}
-        else:
-            deserialized_instance = redis_root.get(self.__class__, id=self.id.value)[0]
-        return deserialized_instance
+        saved_instance = self._set_fields()
+        return saved_instance
 
     def set(self, **fields_with_values):
         name = self.__model_data__['name']
